@@ -152,10 +152,27 @@ typeRepToType tr = do
 -- Without 'ExpLifter', 'lift' tends to just generate code involving
 -- data construction. With 'ExpLifter', you can put more complicated
 -- expression into this construction.
-data ExpLifter = ExpLifter ExpQ deriving (Typeable)
+--
+-- Note that this cannot be used in typed quotes, because 'liftTyped'
+-- will throw an exception. This is because this hack is incompatible
+-- with the type of 'liftTyped', as it would require the generated
+-- code to have type 'ExpLifter'.
+data ExpLifter = ExpLifter
+#if __GLASGOW_HASKELL__ >= 811
+  (forall m. Quote m => m Exp)
+#else
+  ExpQ
+#endif
+  deriving (Typeable)
 
 instance Lift ExpLifter where
   lift (ExpLifter e) = e
+#if MIN_VERSION_template_haskell(2,16,0)
+  liftTyped = error $ concat
+    [ "'liftTyped' is not implemented for 'ExpLifter', "
+    , "because it would require the generated code to have type 'ExpLifter'"
+    ]
+#endif
 
 -- | Print splices generated by a TH splice (the printing will happen
 -- during compilation, as a GHC warning). Useful for debugging.
